apiVersion: v1
kind: Secret
metadata:
  name: truefoundry-helm-repo
  namespace: argocd
  labels:
    argocd.argoproj.io/secret-type: repository
stringData:
  url: {{ tfy_helm_repo_url }}
  name: truefoundry-helm-repo
  type: helm
  password: {{ tfy_helm_repo_token }}
  username: github
